package MIDlets;

import java.util.*;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;

/**
 * MIDlet Record takes in user notes and stores them with timestamps
 * in a RMS record store.
 **/
public class Record extends MIDlet implements CommandListener {

  private Display display;

  // Textbox to take in user input.
  private TextBox textBox;

  // Two on-screen command buttons.
  private Command doneCommand;
  private Command saveCommand;

  // Record store to store notes and timestamps.
  private RecordStore noteStore;

  public Record () throws Exception {

    display = Display.getDisplay(this);

    doneCommand = new Command("DONE", Command.SCREEN, 1);
    saveCommand = new Command("SAVE", Command.SCREEN, 1);

    // If "JWTestStore" does not exist, it opens a new one.
    //
    // Accessing the RMS record store by name ensures all MIDlets
    // use the same record store.
    noteStore = RecordStore.openRecordStore("JWTestStore", true);

  }

  /**
   * Setup the initial display.
   **/
  public void startApp() {
    textBox = new TextBox("Your note", "", 2500, TextField.ANY);

    textBox.addCommand(doneCommand);
    textBox.addCommand(saveCommand);

    // The event handler for the GUI components in this MIDlet
    // is itself.
    textBox.setCommandListener( (CommandListener) this);

    display.setCurrent(textBox);
  }

  public void pauseApp() {
  }

  public void destroyApp(boolean unconditional) {
    // Close the record store before we exit.
    try {
      noteStore.closeRecordStore();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void commandAction(Command command, Displayable screen) {

    if (command == doneCommand) {
      // Do nothing if "DONE" command button is pressed.

      destroyApp(false);
      notifyDestroyed();
    } else if (command == saveCommand) {
      // "SAVE" command saves the current note into the RMS record store.

      // Get the current timestamp
      Calendar cal = Calendar.getInstance();
      String dateString = Long.toString(cal.getTime().getTime());

      // A status message is displayed on a text ticker 
      // after the message is saved successfully.
      String msg = "Last saved at " + 
                   cal.get(Calendar.HOUR_OF_DAY) + ":" +
                   cal.get(Calendar.MINUTE) + ":" +
                   cal.get(Calendar.SECOND) + " " +
                   cal.get(Calendar.YEAR) + "-" +
                   cal.get(Calendar.MONTH) + "-" +
                   cal.get(Calendar.DATE);

      // Get the note text.
      String note = textBox.getString();


      // Store the timestamp and note in the record store.
      try {
        noteStore.addRecord(dateString.getBytes(), 0, dateString.length());
        noteStore.addRecord(note.getBytes(), 0, note.length());
      } catch (Exception e) {
        msg = "Error in Saving!";
      }

      // Make a ticker.
      Ticker ticker = new Ticker( msg );
      
      // Start a new screen.
      textBox = new TextBox("Your note:", "", 2500, TextField.ANY);
      textBox.addCommand(doneCommand);
      textBox.addCommand(saveCommand);
      // Text ticker for the info from the last save.
      textBox.setTicker(ticker);
      textBox.setCommandListener( (CommandListener) this);
      display.setCurrent(textBox);
    }
  }

}
