package MIDlets;

import java.io.*;
import java.util.*;
import support.*;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import javax.microedition.rms.*;

/**
 * MIDlet SendData sends all the records in the on-device record store
 * to a JavaServer Page.
 **/
public class SendData extends MIDlet implements CommandListener {
   
  // URL of the database update JSP page.
  private String url = "http://localhost:8080/JWTest/updateDB.jsp";

  private Display display;

  // Three on-screen command buttons.
  private Command cancelCommand;
  private Command doneCommand;
  private Command sendCommand;

  // Record store that stores notes and timestamps.
  private RecordStore noteStore;

  public SendData() throws Exception {

    display = Display.getDisplay(this);
    cancelCommand = new Command("CANCEL", Command.CANCEL, 1);
    doneCommand = new Command("DONE", Command.CANCEL, 1);
    sendCommand = new Command("SEND", Command.SCREEN, 1);

    // Accessing the RMS record store by name ensures we use the same
    // record store as the one in MIDlet Record.
    noteStore = RecordStore.openRecordStore("JWTestStore", true);
  }

  /**
   * Setup intial information screen.
   **/
  public void startApp() {

    String notice = null;

    try {
      // Half of the records are timestamp fields.
      int numRec = noteStore.getNumRecords() / 2;
      notice = Integer.toString( numRec) +
               " unsent notes. \n\n Do you want to send them to the " +
               " database now? \n";
    } catch (Exception e) {
      notice = "Cannot open any database";
      e.printStackTrace();
    }

    // New form contains the message (number of notes) and
    // three buttons. 
    // The event handler is this class itself.
    Form form = new Form("Send Data");
    StringItem sItem = new StringItem("", notice);
    form.append( sItem );
    form.addCommand(cancelCommand);
    form.addCommand(sendCommand);
    form.setCommandListener( (CommandListener) this);
    display.setCurrent(form);

  }

  public void pauseApp() {
  }

  public void destroyApp(boolean unconditional) {
    // Close the record store before we exit.
    try {
      noteStore.closeRecordStore();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void commandAction(Command command, Displayable screen) {

    if (command == cancelCommand) {
      // Do nothing if "CANCEL" button is pressed.

      destroyApp(false);
      notifyDestroyed();
    } else if (command == doneCommand) {
      // "DONE" is the same as "CANCEL"

      destroyApp(false);
      notifyDestroyed();
    } else if (command == sendCommand) {
      // "SEND" command sends data to the server and displays 
      // the response from the server.

      // "status" is displayed after the data is sent.
      // The following message will be over-written 
      // if the update is successful.
      String status = "not defined";

      try {

        // Open a session-aware HTTP connection to the
        // desired JSP page.
        HttpConnection conn = SessionConnector.open( url );
        conn.setRequestMethod(HttpConnection.POST);

        // New XML data document to be sent to the JSP page.
        StringBuffer postBuffer = new StringBuffer();
        postBuffer.append("<root>");

        // Row name and field names in the DB format XML document.
        String tableName = "Notes";
        String [] fieldNames = {"Time", "NoteText"};

        // Get all records from the RMS record store
        RecordEnumeration re = noteStore.enumerateRecords(null, null, false);
        int index = 0, id;
        String timeStr = null, noteStr = null;
        while ( re.hasNextElement() ) {
          // Get the ID of the next record.
          id = re.nextRecordId();
          // Get the content of the record.
          byte [] b = noteStore.getRecord(id);

          if ( index == 0 ) {
            // timestamp field
            timeStr = new String (b);
            index++;
          } else {
            // note text field.
            if ( b != null ) {
              noteStr = new String (b);
            } else {
              noteStr = "";
            }
            index = 0;

            // Use the DBXMLParser.rowString() utility method to
            // assemble a correct XML segment from the
            // timestamp and note just retrieved.
            //
            // Add the above XML segment to the XML document to be sent
            // to the server.
            String [] fieldValues = {timeStr, noteStr};
            postBuffer.append( DBXMLParser.rowString(tableName, fieldNames, fieldValues) );
          }
          // Delete a record after it is retrieved successfully.
          // In real world, I should probablly delete it after I 
          // have checked the response from the server. But I do it 
          // this way here for simplicity.
          noteStore.deleteRecord(id);
        }
        // Close the root tag.
        postBuffer.append("</root>");
        String post = postBuffer.toString();

        // Send the XML document to the JSP page through
        // a HTTP request.
        DataOutputStream os = conn.openDataOutputStream();
        os.write(post.getBytes());
        os.flush();
        os.close();

        // Get the response from the JSP page.
        DataInputStream is = conn.openDataInputStream();
        DBXMLParser p = new DBXMLParser( is );
        // Get the status and over-write the "not defined" value.
        status = p.getFieldValue("Response", "Status", 0);
        is.close();

        // important!
        conn.close();

      } catch (Exception e) {
        status = "failed";
        e.printStackTrace();
      }

      // Result screen.
      Form form = new Form("Results");
      StringItem sItem;
      // Display the status returned from the server.
      sItem  = new StringItem("Status", status.trim() );
      form.append( sItem );
      form.addCommand(doneCommand);
      form.setCommandListener( (CommandListener) this);
      display.setCurrent(form);

    } 
  }

}
