package MIDlets;

import java.io.*;
import java.util.*;
import support.*;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;

/**
 * MIDlet Session updates user session information (username) 
 * on the server side.
 **/
public class Session extends MIDlet implements CommandListener {

  private String username;

  // URL to get session information.
  private String getURL = "http://localhost:8080/JWTest/getSession.jsp";
  // URL to update session information.
  private String setURL = "http://localhost:8080/JWTest/updateSession.jsp";

  private Display display;

  // Three on-screen command buttons.
  private Command cancelCommand;
  private Command doneCommand;
  private Command sendCommand;

  // TextField allows user to input username string.
  private TextField textField;

  public Session () throws Exception {

    display = Display.getDisplay(this);
    cancelCommand = new Command("CANCEL", Command.CANCEL, 1);
    doneCommand = new Command("DONE", Command.CANCEL, 1);
    sendCommand = new Command("SEND", Command.SCREEN, 1);

  }

  /**
   * Get the current username from the server and display it.
   **/
  public void startApp() {

    try {

      // Send a session-aware request. 
      HttpConnection conn = SessionConnector.open( getURL );
      conn.setRequestMethod(HttpConnection.POST);

      // Get the response from the server.
      //
      // If there is already a session associated with the
      // the current connection, the username is returned
      // for the user to verify.
      //
      // If this is a new connection, string "anonymous"
      // is returned. Please refer to the source code on the 
      // server side.
      DataInputStream is = conn.openDataInputStream();
      DBXMLParser p = new DBXMLParser( is );
      // Get username info from the response XML.
      username = p.getFieldValue("SessionInfo", "Username", 0);
      is.close();
      username = username.trim();

      // important!
      conn.close();

    } catch (Exception e) {
      username = "There is an ERROR!";
      e.printStackTrace();
    }

    // Display the username in an editable TextField
    // with command buttons. 
    Form form = new Form("Change Session Info");    
    textField = new TextField("Username:", username,
                              20, TextField.ANY);
    form.append( textField );
    form.addCommand(cancelCommand);
    form.addCommand(sendCommand);
    form.setCommandListener( (CommandListener) this);

    display.setCurrent(form);
  }

  public void pauseApp() {
  }

  public void destroyApp(boolean unconditional) {
  }

  public void commandAction(Command command, Displayable screen) {

    if (command == cancelCommand) {
      // Do nothing if "CANCEL" is pressed

      destroyApp(false);
      notifyDestroyed();
    } else if (command == doneCommand) {
      // "DONE" is the same as "CANCEL"

      destroyApp(false);
      notifyDestroyed();
    } else if (command == sendCommand) {
      // "SEND" command sends data to the server and displays 
      // response from the server.

      // "status" is displayed after the data is sent.
      // The following message will be over-written 
      // if the update is successful.
      String status = "not defined";

      try {

        // Open a session-aware HTTP connection to the
        // desired JSP page.
        HttpConnection conn = SessionConnector.open( setURL );
        conn.setRequestMethod(HttpConnection.POST);

        // Get the new username from the TextField box.
        username = textField.getString();
        String tableName = "SessionInfo";
        String [] fieldNames = {"Username"};
        String [] fieldValues = {username};
        // And assemble it into an XML document to be sent to the server.
        String post = "<root>" + 
                    DBXMLParser.rowString(tableName, fieldNames, fieldValues) +
                      "</root>\n";

        // Send the XML document containing username info to the 
        // server through a HTTP request.
        DataOutputStream os = conn.openDataOutputStream();
        os.write(post.getBytes());
        os.flush();
        os.close();

        // Get the response from the JSP page.
        DataInputStream is = conn.openDataInputStream();
        DBXMLParser p = new DBXMLParser( is );
        // Get the status and over-write the "not defined" value.
        status = p.getFieldValue("Response", "Status", 0);
        is.close();

        // important!
        conn.close();

      } catch (Exception e) {
        status = "failed";
        e.printStackTrace();
      }

      // Display new username in the TexField with command buttons.
      Form form = new Form("Results");    
      StringItem sItem;
      sItem  = new StringItem("Status", status.trim() );
      form.append( sItem );
      form.addCommand(doneCommand);
      form.setCommandListener( (CommandListener) this);
      display.setCurrent(form);
    }
  }

}
